@@ -5,10 +5,13 @@
 ##       fixes them in all functions simultaneously.
 ##
 ## updated by ADM 7/22/04
-## re-organized (alphabetical) by adm 7/28/04
+## re-organized (alphabetical) by ADM 7/28/04
+## added a number of functions for teaching models by ADM 1/25/2006 
 
 ## create an agreement score matrix from a vote matrix
 ## subjects initially on rows and items on cols of X
+## note: treats missing votes as category for agreement / might be
+##       more principled to treat them in another fashion
 "agree.mat" <- function(X){
   X <- t(X) # put subjects on columns
   n <- ncol(X)
@@ -99,6 +102,66 @@
     return(0)
   }
 
+# check beta prior
+"check.beta.prior" <-
+   function(alpha, beta) {
+     
+    if(alpha <= 0) {
+      cat("Error: Beta(alpha,beta) prior alpha less than or equal to zero.\n")
+      stop("Please respecify and call ", calling.function(), " again.\n",
+         call.=FALSE)
+    }
+    if(beta <= 0) {
+      cat("Error: Beta(alpha,beta) prior beta less than or equal to zero.\n")
+       stop("Please respecify and call ", calling.function(), " again.\n",
+        call.=FALSE)    
+    }
+    return(0)
+  }
+
+# check Gamma prior
+# ADM 1/25/2006
+"check.gamma.prior" <-
+   function(alpha, beta) {
+     
+    if(alpha <= 0) {
+      cat("Error: Gamma(alpha,beta) prior alpha less than or equal to zero.\n")
+      stop("Please respecify and call ", calling.function(), " again.\n",
+         call.=FALSE)
+    }
+    if(alpha <= 0) {
+      cat("Error: Gamma(alpha,beta) prior beta less than or equal to zero.\n")
+       stop("Please respecify and call ", calling.function(), " again.\n",
+        call.=FALSE)    
+    }
+    return(0)
+  }   
+
+# check Normal prior
+# ADM 1/26/2006
+"check.normal.prior" <-
+   function(mu, sigma2) {
+   
+     if(sigma2 <= 0) {
+       cat("Error: Normal(mu0,tau20) prior sigma2 less than or equal to zero.\n")
+       stop("Please respecify and call ", calling.function(), " again.\n",
+         call.=FALSE)    
+     }
+   }
+
+# check mc parameter
+# ADM 1/25/2006
+"check.mc.parameter" <-
+  function(mc) {
+  
+    if(mc < 0) {
+      cat("Error: Monte Carlo iterations negative.\n")
+      stop("Please respecify and call ", calling.function(), " again.",
+         call.=FALSE) 
+    }   
+    return(0)
+  }
+
 # check mcmc parameters
 "check.mcmc.parameters" <-
   function(burnin, mcmc, thin) {
@@ -131,7 +194,8 @@
    function(args) {
       if(sum(names(args)=="offset")==1) {
          cat("Error: Offsets are currently not supported in MCMCpack.\n")
-         stop("Please respecify and call ", calling.function(), " again.\n")
+         stop("Please respecify and call ", calling.function(), " again.\n",
+            call.=FALSE)
       }
    return(0)
    }
@@ -141,7 +205,7 @@
 #       or for another model by passing default starting values to
 #       the function   
 "coef.start" <-
-   function(beta.start, K, formula, family, data, defaults=NA) {
+   function(beta.start, K, formula, family, data=NULL, defaults=NA) {
           
      if (is.na(beta.start)[1] & is.na(defaults)[1]){ # use GLM estimates
        beta.start <- matrix(coef(glm(formula, family=family, data=data)), K, 1)
@@ -488,17 +552,28 @@
 
 # pull together the posterior density sample
 "form.mcmc.object" <-
-  function(posterior.object, names, title) {
+  function(posterior.object, names, title, ...) {
     holder <- matrix(posterior.object$sampledata,
                      posterior.object$samplerow,
                      posterior.object$samplecol,
-                     byrow=TRUE)
-    
+                     byrow=FALSE)
+      
     output <- mcmc(data=holder, start=(posterior.object$burnin+1),
                    end=(posterior.object$burnin+posterior.object$mcmc),
                    thin=posterior.object$thin)
     varnames(output) <- as.list(names)
     attr(output,"title") <- title
+    
+    attribs <- list(...)
+    K <- length(attribs)
+    attrib.names <- names(attribs)
+
+    if (K>0){
+      for (i in 1:K){
+        attr(output, attrib.names[i]) <- attribs[[i]]
+      }
+    }
+    
     return(output)  
   }
 
@@ -518,6 +593,9 @@
      
      # prior precision
      if(is.null(dim(B0))) {
+       if (length(B0) > K){
+         stop("B0 was passed as a vector longer than K.\nB0 must be either a scalar or a matrix.\nPlease respecify and call ", calling.function(), " again.\n", call.=FALSE)
+       }
        B0 <- B0 * diag(K)    
      }
      if((dim(B0)[1] != K) || (dim(B0)[2] != K)) {
@@ -525,6 +603,22 @@
        stop("Please respecify and call ", calling.function(), " again.\n",
          call.=FALSE)
      }
+
+     ## check B0 for symmetry
+     symproblem <- FALSE
+     for (i in 1:K){
+       for (j in i:K){
+         if (B0[i,j] != B0[j,i]){
+           symproblem <- TRUE
+         }
+       }
+     }
+     if (symproblem){
+       cat("B0 is not symmetric.\n")
+       stop("Please respecify and call ", calling.function(), " again.\n",
+         call.=FALSE)       
+     }
+     
      return(list(b0,B0))
    }
 
@@ -569,7 +663,7 @@
            stop("Please respecify and call ", calling.function(), " again.",
                 call.=FALSE)          
          }
-         if( all(lec.seed[1:3]) == 0 ){
+         if( all(lec.seed[1:3] == 0 )){
            cat("Error: first three L'Ecuyer seeds == 0\n")
            stop("Please respecify and call ", calling.function(), " again.",
                 call.=FALSE)          
@@ -580,7 +674,7 @@
            stop("Please respecify and call ", calling.function(), " again.",
                 call.=FALSE)          
          }         
-         if( all(lec.seed[4:6]) == 0 ){
+         if( all(lec.seed[4:6] == 0 )){
            cat("Error: last three L'Ecuyer seeds == 0\n")
            stop("Please respecify and call ", calling.function(), " again.",
                 call.=FALSE)          
@@ -625,16 +719,15 @@
 # parse formula and return a list that contains the model response
 # matrix as element one, and the model matrix as element two
 "parse.formula" <- 
-   function(formula, data, intercept=TRUE, justX=FALSE) {
+   function(formula, data=NULL, intercept=TRUE, justX=FALSE) {
 
     # extract Y, X, and variable names for model formula and frame
-    mt <- terms(formula, data=data)
-    if(missing(data)) data <- sys.frame(sys.parent())
     mf <- match.call(expand.dots = FALSE)
     mf$intercept <- mf$justX <- NULL
     mf$drop.unused.levels <- TRUE
     mf[[1]] <- as.name("model.frame")
     mf <- eval(mf, sys.frame(sys.parent()))
+    mt <- attr(mf, "terms")
     if (!intercept){
       attributes(mt)$intercept <- 0
     }
